/*---------------------------------------------------------------------------*
 *
 * Project:	fXPlayMidi
 *
 * $RCSfile: album.c,v $
 * $Date: 1997/09/14 01:25:59 $
 * $Author: klmcw $
 * $Revision: 1.9 $
 *
 *---------------------------------------------------------------------------*
 * $Log: album.c,v $
 * Revision 1.9  1997/09/14 01:25:59  klmcw
 * Broke up albumRead into albumRead and albumReadPath to accomodate reading album
 * from file specified on command line.
 *
 * Revision 1.8  1997/04/26 08:13:02  klmcw
 * Set browser font size/style to something sane!! Finally!
 *
 * Revision 1.7  1997/04/26 07:40:51  klmcw
 * fl_show_question call still not formed correctly in albumRemove_cb.
 * Sheesh it's getting late :)
 *
 * Revision 1.6  1997/04/26 07:33:22  klmcw
 * Made 0.86 changes to fl_show_question calls.
 *
 * Revision 1.5  1996/04/19 13:13:23  klmcw
 * Initial port to libforms.so.0.81
 *
 * Revision 1.4  1996/03/19 13:54:12  klmcw
 * Made albumFindPath() visible across all modules.
 *
 * Revision 1.3  1996/03/09 08:20:18  klmcw
 * Added firsttime special case to prompt for options when they don't exist yet.
 * Added code to save album upon exit if it has changed.
 *
 * Revision 1.2  1996/03/02 19:59:44  klmcw
 * Added #includes per -Wall suggestions -- better lint than lint!
 *
 * Revision 1.1  1996/02/26 02:22:40  klmcw
 * Initial revision
 *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/

#include <limits.h>
#include <memory.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>
#include <unistd.h>

#include "album.h"
#include "file.h"
#include "options.h"
#include "path.h"

/*---------------------------------------------------------------------------*/

static void albumRemove_cb(void *);

/*---------------------------------------------------------------------------*/

static char CurrAlbum[_POSIX_PATH_MAX+1];
static int Modified = 0;

/*---------------------------------------------------------------------------*/
FL_OBJECT *
albumBrp(XPMApp_t *App)
{
  return(App->Album->brAlbumContents);
}
/*---------------------------------------------------------------------------*/
const char *
albumCurrentGet(void)
{
  return(CurrAlbum);
}
/*---------------------------------------------------------------------------*/
void
albumCurrentSet(const char *Alb)
{
  if((Alb != NULL) && *Alb) strcpy(CurrAlbum, Alb);
}
/*---------------------------------------------------------------------------*/
void
albumFileAdd(XPMApp_t *App, const char *FileNm)
{
  if(FileNm != NULL) {
    fl_addto_browser(albumBrp(App), FileNm);

    Modified = 1;
  }
}
/*---------------------------------------------------------------------------*/
void
albumFileRemove(XPMApp_t *App, int Ln)
{
  fl_delete_browser_line(albumBrp(App), Ln);

  Modified = 1;
}
/*---------------------------------------------------------------------------*/
int
albumIsModified(void)
{
  if(Modified && fl_show_question("Album is Modified! Save Changes ?", 1))
    return(1);

  return(0);
}
/*---------------------------------------------------------------------------*/
const char *
albumLineGet(XPMApp_t *App, int Ln)
{
  return(fl_get_browser_line(albumBrp(App), Ln));
}
/*---------------------------------------------------------------------------*/
int
albumLineNumCurr(XPMApp_t *App)
{
  return(fl_get_browser(albumBrp(App)));
}
/*---------------------------------------------------------------------------*/
int
albumLineNumNext(XPMApp_t *App)
{
  register int Ln;

  /* Random Mode */
  if(optionsRandomGet())
    return(albumLineNumRandom(App));

  /* Sequential Mode */
  if((Ln = albumLineNumCurr(App)) > 0)
    if(++Ln > fl_get_browser_maxline(albumBrp(App)))
      if(optionsRepeatGet())
        Ln = 1;
      else
        Ln = -1;

  return(Ln);
}
/*---------------------------------------------------------------------------*/
int
albumLineNumRandom(XPMApp_t *App)
{
  register int Max = fl_get_browser_maxline(albumBrp(App));

  return((Max) ? (rand() % Max) + 1 : 0);
}
/*---------------------------------------------------------------------------*/
const char *
albumPathFind(XPMApp_t *App)
{
  static char AlbPath[_POSIX_PATH_MAX+1];
  const char *Path = pathFind();

  if(Path != NULL) {
    memset(AlbPath, '\0', sizeof(AlbPath));
    sprintf(AlbPath, "%s/Albums", Path);
    if(access(AlbPath, R_OK|W_OK) < 0)
      mkdir(AlbPath, 0755);

    return(AlbPath);
  }

  return(NULL);
}
/*---------------------------------------------------------------------------*/
void
albumPlay(XPMApp_t *App)
{
  register int Ln = (fileIsPlaying()) ? 1 : albumLineNumCurr(App);

  if((Ln <= 0) && optionsRandomGet())
    if((Ln = albumLineNumRandom(App))) {
      fl_deselect_browser(albumBrp(App));
      fl_select_browser_line(albumBrp(App), Ln);
      fl_set_browser_topline(albumBrp(App), Ln);
    }

  if(Ln > 0) {
    filePlay(App, Ln);
  } else
    fl_show_alert("Select a file to play first", "", "", 1);
}
/*---------------------------------------------------------------------------*/
int
albumRead(XPMApp_t *App, const char *Album)
{
  const char *Path = albumPathFind(App);
  char AlbumPath[_POSIX_PATH_MAX+1];

  if(Path == NULL) {
    fl_show_alert("Could not open Albums directory!","","",1);
    return(-1);
  }
  memset(AlbumPath, '\0', sizeof(AlbumPath));
  sprintf(AlbumPath, "%s/%s", Path, Album);

  return(albumReadPath(App, AlbumPath));
}
/*---------------------------------------------------------------------------*/
int
albumReadPath(XPMApp_t *App, const char *AlbumPath)
{
  register int rc = 0;
  char *Album, albumLine[_POSIX_PATH_MAX+1], *cp;
  FILE *Fp;

  /*fl_show_alert("AlbumPath is set to", AlbumPath, "", 1);*/

  if((Fp = fopen(AlbumPath, "r")) == NULL) {
    if(!optionsFirstTime())
      fl_show_alert("Could not open Album!", "", AlbumPath, 1);
    rc = -1;
  } else {
    if(albumIsModified()) albumWrite(App);

    fl_freeze_form(App->Album->fmAlbum);
    fl_clear_browser(albumBrp(App));
    fl_set_browser_fontsize(albumBrp(App), FL_MEDIUM_SIZE);
    fl_set_browser_fontstyle(albumBrp(App), FL_FIXED_STYLE);

    while(fgets(albumLine, sizeof(albumLine), Fp) != NULL) {
      if(albumLine[0] == '#') continue;

      if((cp = strchr(albumLine, '\n')) != NULL) *cp = '\0';
      albumFileAdd(App, albumLine);
    }
    fl_unfreeze_form(App->Album->fmAlbum);

    fclose(Fp);
  }

  if(!rc) {
    Modified = 0;
    if((Album = strrchr(AlbumPath, '/')) != NULL) {
      albumCurrentSet(++Album);
      srand((unsigned int)time(NULL));
    } else
      rc = -1;
  }

  return(rc);
}
/*---------------------------------------------------------------------------*/
static void
albumRemove_cb(void *ptr)
{
  const char *Albumcp = fl_get_filename();
  char chAlbumPath[_POSIX_PATH_MAX+1];

  if(!strcmp(Albumcp, CurrAlbum)) {
    fl_show_alert("Cannot Remove Current Album", "", "", 1);
    return;
  }

  memset(chAlbumPath, '\0', sizeof(chAlbumPath));
  sprintf(chAlbumPath, "Remove Album [%s] Are You Sure ?", Albumcp);
  if(fl_show_question(chAlbumPath, 0)) {
    memset(chAlbumPath, '\0', sizeof(chAlbumPath));
    sprintf(chAlbumPath, "%s/%s", albumPathFind(NULL), Albumcp);
    unlink(chAlbumPath);
    fl_refresh_fselector();
  }
}
/*---------------------------------------------------------------------------*/
void
albumSelect(XPMApp_t *App, int Read)
{
  char Curr[_POSIX_PATH_MAX+1];
  register int Done = 0;

  memset(Curr, '\0', sizeof(Curr));
  strcpy(Curr, albumCurrentGet());

  fl_add_fselector_appbutton("Remove", albumRemove_cb, NULL);

  while(!Done)
    if(fl_show_fselector("fXPlayMidi Load Album", albumPathFind(App), "*", NULL) != NULL)
      if(Read) {
	if(albumRead(App, fl_get_filename()) >= 0)
	  Done = 1;
      } else {
	fl_set_input(App->Options->inOptionsAlbum, fl_get_filename());
	Done = 1;
      }
    else {
      albumCurrentSet(Curr);
      Done = 1;
    }

  fl_remove_fselector_appbutton("Remove");
}
/*---------------------------------------------------------------------------*/
int
albumWrite(XPMApp_t *App)
{
  register int Ln;
  const char *cp, *Path = albumPathFind(App);
  char AlbumPath[_POSIX_PATH_MAX+1];
  FILE *Fp;

  if(Path == NULL) return(-1);
  memset(AlbumPath, '\0', sizeof(AlbumPath));
  sprintf(AlbumPath, "%s/%s", Path, CurrAlbum);

  if((Fp = fopen(AlbumPath, "w")) == NULL) return(-1);

  for(Ln = 0; Ln <= fl_get_browser_maxline(albumBrp(App)); Ln++)
    if((cp = albumLineGet(App, Ln)) != NULL)
      fprintf(Fp, "%s\n", cp);

  fclose(Fp);
  Modified = 0;

  return(0);
}
/*---------------------------------------------------------------------------*/
